#include <types.h>
#include <types.h>

__global__ void collision(TYPE* gpu_c1, TYPE* gpu_c2, TYPE* gpu_c3,
			  TYPE* gpu_c4, TYPE* gpu_c5, TYPE* gpu_c6,
			  TYPE* gpu_k1, TYPE* gpu_k2, TYPE* gpu_k3,
			  TYPE* gpu_k4, TYPE* gpu_k5, TYPE* gpu_k6,
			  TYPE* gpu_boundary) {
	int x = blockIdx.x * blockDim.x + threadIdx.x; 
	int y = blockIdx.y * blockDim.y + threadIdx.y; 
	int index = y * LATTICE_HEIGHT + x;
	if(index > LATTICE_WIDTH * LATTICE_HEIGHT) {
		return;
	}
	unsigned int ro;

	TYPE a = gpu_c1[index];
	TYPE b = gpu_c2[index];
	TYPE c = gpu_c3[index];
	TYPE d = gpu_c4[index];
	TYPE e = gpu_c5[index];
	TYPE f = gpu_c6[index];

	TYPE tripple = (a ^ b) & (b ^ c) & (c ^ d) & (d ^ e) & (e ^ f);

	TYPE db1  = (a & d & ~(b | c | e | f));
	TYPE db2  = (b & e & ~(a | c | d | f));
	TYPE db3  = (c & f & ~(a | b | d | e));

	ro        = 0; //flip_coin( );

	gpu_k1[index]= a ^ ((tripple | db1 | (ro & db2) | (~ro & db3)) & gpu_boundary[index]);
	gpu_k2[index]= b ^ ((tripple | db2 | (ro & db1) | (~ro & db3)) & gpu_boundary[index]);
	gpu_k3[index]= c ^ ((tripple | db3 | (ro & db1) | (~ro & db2)) & gpu_boundary[index]);
	gpu_k4[index]= d ^ ((tripple | db1 | (ro & db2) | (~ro & db3)) & gpu_boundary[index]);
	gpu_k5[index]= e ^ ((tripple | db2 | (ro & db1) | (~ro & db3)) & gpu_boundary[index]);
	gpu_k6[index]= f ^ ((tripple | db3 | (ro & db1) | (~ro & db2)) & gpu_boundary[index]);
}

__global__ void propagate(TYPE* gpu_c1, TYPE* gpu_c2, TYPE* gpu_c3,
			  TYPE* gpu_c4, TYPE* gpu_c5, TYPE* gpu_c6,
			  TYPE* gpu_k1, TYPE* gpu_k2, TYPE* gpu_k3,
			  TYPE* gpu_k4, TYPE* gpu_k5, TYPE* gpu_k6) {
	int x = blockIdx.x * blockDim.x + threadIdx.x; 
	int y = blockIdx.y * blockDim.y + threadIdx.y; 
	int index = y * LATTICE_HEIGHT + x;

	if(index > LATTICE_WIDTH * LATTICE_HEIGHT) {
		return;
	}

	gpu_c1[((x + 1 + LATTICE_WIDTH) % LATTICE_WIDTH) + LATTICE_WIDTH*((y     + LATTICE_HEIGHT) % LATTICE_HEIGHT)] = gpu_k1[index];
	gpu_c2[((x     + LATTICE_WIDTH) % LATTICE_WIDTH) + LATTICE_WIDTH*((y + 1 + LATTICE_HEIGHT) % LATTICE_HEIGHT)] = gpu_k2[index];
	gpu_c3[((x - 1 + LATTICE_WIDTH) % LATTICE_WIDTH) + LATTICE_WIDTH*((y + 1 + LATTICE_HEIGHT) % LATTICE_HEIGHT)] = gpu_k3[index];
	gpu_c4[((x - 1 + LATTICE_WIDTH) % LATTICE_WIDTH) + LATTICE_WIDTH*((y     + LATTICE_HEIGHT) % LATTICE_HEIGHT)] = gpu_k4[index];
	gpu_c5[((x     + LATTICE_WIDTH) % LATTICE_WIDTH) + LATTICE_WIDTH*((y - 1 + LATTICE_HEIGHT) % LATTICE_HEIGHT)] = gpu_k5[index];
	gpu_c6[((x + 1 + LATTICE_WIDTH) % LATTICE_WIDTH) + LATTICE_WIDTH*((y - 1 + LATTICE_HEIGHT) % LATTICE_HEIGHT)] = gpu_k6[index];
}
